package PLex::Context;

use strict;
use PLex;
use vars qw($VERSION @ISA);

require AutoLoader;

@ISA = qw(PLex AutoLoader);
$VERSION = '0.01';

# Preloaded methods
sub new {
    my $class = shift;
    my $self = PLex->new;
    bless $self, $class;
    return $self;
}

# AUTOLOAD stubs
sub output;
sub output_state;

# 1;
# __END__

=head1 NAME

PLex::Context - abstraction of minimal system services for parsers

=head1 SYNOPSIS

  use PLex::Context;

=head1 DESCRIPTION

PLex::Context is a base class for "system services." In this minimal
implementation, this just means being able to append to an output
string and get the contents of said string.

=head1 METHODS

The PLex::Context object provides the following methods:

=over 4

=item *

new()

I<Usage>

    $context = new PLex::Context;

I<Description>

Creates a new Context object with an empty output

=item *

output()

I<Usage>

    $context->output("some output");

I<Description>

Adds the passed string to the output

=item *

output_state()

I<Usage>

    print $context->output_state();

I<Description>

returns a string containing all the context appended using output()

=head1 AUTHOR

Kyle Downey, kdowney@xline.com

=head1 COPYRIGHT

Copyright 1997, X-Line Software. This software is freely-redistributable
under the terms of the GNU General Public License

=head1 SEE ALSO

PLex(3), PLex::Lexer(3), PLex::Parser(3), PLex::Namespace(3)

=cut

# return a reference to a string where all the output is stored
sub output_state {
    return shift->{'_output'};
}

# adds the reference scalar to the output string
sub output {
    my ($self, $str) = @_;
    $self->{'_output'} .= $str;
}

1;

__END__
