package PLex;

use strict;
use vars qw($VERSION @ISA);
require AutoLoader;

@ISA = qw(AutoLoader);
$VERSION = '0.01';

# pre-loaded methods
sub new {
    my ($class) = @_;
    my $self = { };
    bless $self, $class;

    $self->{'_error'} = 'none';
    return $self;
}

sub error {
    my ($self, $err_str) = @_;
    $self->{'_error'} = $err_str;
    return -1;
}

sub strerror {
    return shift->{'_error'};
}

1;
__END__

=head1 NAME

PLex - base class for the Perl-Lexer (PLex) suite

=head1 SYNOPSIS

  use PLex;

=head1 DESCRIPTION

The PLex module provides base facilities--for error logging and
reporting--for all of the components of the lexer-parser

=head1 METHODS

The PLex object provides the following methods:

=over 4

=item *

new()

I<Usage>

    $plex = new PLex;

I<Description>

The constructor sets the error string to 'none;' it is used internally
by child objects.

=item *

error()

I<Usage>

    $plex->error("an error");

I<Description>

Sets the internal error string

=item *

strerror()

I<Usage>

    print $plex->strerror();

I<Description>

returns the most recent error string recorded; 

=back

=head1 AUTHOR

Kyle Downey, kdowney@xline.com

=head1 COPYRIGHT

Copyright 1997, X-Line Software. This software is freely-redistributable
under the terms of the GNU General Public License

=head1 SEE ALSO

PLex::Lexer(3), PLex::Parser(3), PLex::Context(3), PLex::Namespace(3), PLex::Actions.

=cut
