/* $Id: misc.c,v 1.6 1998/02/11 03:01:00 xach Exp $ 
** $Source: /usr/local/cvs/snarf/misc.c,v $
*/

#include "config.h"
#include <sys/types.h>
#include <sys/socket.h>

#ifdef HAVE_SYS_FILE_H
# include <sys/file.h>
#endif

#ifdef HAVE_SYS_FCNTL_H
# include <sys/fcntl.h>
#endif

#include <sys/stat.h>
#include <netinet/in.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <stdio.h>
#include "protocol.h"
#include "url.h"
#include "option.h"
#include "misc.h"

#ifndef HAVE_HERROR
extern int h_errno;
#endif

#define BASE64_END 	'='


#define BIG_BUFFER_SIZE 4096

const char base64_table[64]= "ABCDEFGHIJKLMNOPQRSTUVWXYZ" 
                             "abcdefghijklmnopqrstuvwxyz"
                             "0123456789+/";


/* Reads a \n terminated line (includes the \n) */
int readline (int fd, char *line) 
{
  int n, rc = 0;
  char c ;

  for (n = 1 ; n < BIG_BUFFER_SIZE ; n++)
    {
      if ((rc == read (fd, &c, 1)) != 1)
	{
	  *line++ = c ;
	  if (c == '\n')
	    break ;
	}
      else if (rc == 0)
	{
	  if (n == 1)
	    return 0 ; /* EOF */
	  else
	    break ; /* unexpected EOF */
	}
      else 
	return -1 ;
    }

  *line = 0 ;
  return n ;
}

int dump_data(int fd_in, int fd_out){
  int i;
  char recv_buf[BIG_BUFFER_SIZE];
  
  while((i = read(fd_in, recv_buf, sizeof(recv_buf))))
    write(fd_out, recv_buf, i);

  return(1);
}


int tcp_connect(struct url u) {
  struct hostent *host;
  struct sockaddr_in sa;
  int sock_fd;

  if((host = gethostbyname(u.u_host)) == NULL) {
    my_herror(u.u_host);
    return(0);
  }

  /* get the socket */
  if((sock_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket");
    return(0);
  }

  /* connect the socket, filling in the important stuff */
  sa.sin_family = AF_INET;
  sa.sin_port = htons(u.u_port);
  memcpy(&sa.sin_addr, host->h_addr,host->h_length);
  
  if(connect(sock_fd, (struct sockaddr *)&sa, sizeof(sa)) < 0){
    perror(u.u_host);
    return(0);
  }

  return sock_fd;
}

int open_output(char *outfile)
{
  int fd;

  /* get a file descriptor pointing at the right place */
  if(strcmp(outfile, "-") == 0)
    fd = 1;
  else
    if((fd = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, 0644)) < 0) {
      return(0);
    }

  return fd;
}


void my_herror(char *prefix)
{
  
#ifdef HAVE_HERROR

  herror(prefix);

#else
  char my_error[128];

  switch(h_errno) {
  case HOST_NOT_FOUND:
    strcpy(my_error, "Host not found");
    break;
  case TRY_AGAIN:
    strcpy(my_error, "Host not found, but I am optimistic that if you try"
	   " again it might work");
    break;
  case NO_RECOVERY:
    strcpy(my_error, "Unrecoverable hostname lookup failure. Bummer!");
    break;
  case NO_ADDRESS:
    strcpy(my_error, "No address associated with name");
    break;
  default:
    strcpy(my_error, "Unknown error");
  }

  fprintf(stderr, "%s: %s\n", prefix, my_error);

#endif /* HAVE_HERROR */
}



/* lauri alanko, i thank you */
char *base64(unsigned char *bin, int len){
	char *buf= (char *)malloc((len+2)/3*4+1);
	int i=0, j=0;
	
	while(j<len-2){
		buf[i++]=base64_table[bin[j]>>2];
		buf[i++]=base64_table[((bin[j]&3)<<4)|(bin[j+1]>>4)];
		buf[i++]=base64_table[((bin[j+1]&15)<<2)|(bin[j+2]>>6)];
		buf[i++]=base64_table[bin[j+2]&63];
		j+=3;
	}
	switch(len-j){
	    case 1:
		buf[i++]=base64_table[bin[j]>>2];
		buf[i++]=base64_table[(bin[j]&3)<<4];
		buf[i++]=BASE64_END;
		buf[i++]=BASE64_END;
		break;
	    case 2:
		buf[i++]=base64_table[bin[j]>>2];
		buf[i++]=base64_table[((bin[j]&3)<<4)|(bin[j+1]>>4)];
		buf[i++]=base64_table[(bin[j+1]&15)<<2];
		buf[i++]=BASE64_END;
		break;
	    case 0:
	      break;
	}
	buf[i]='\0';
	return buf;
}

/* this function returns the size of a file (if it exists) or
   zero if it doesnt */
off_t getsize(const char *file){
  struct stat file_info;

  if(stat(file, &file_info) == -1)
    return(0);
  else
    return(file_info.st_size);
}


	
	
	
	

