/* -*- mode: C; c-basic-offset: 8; indent-tabs-mode: nil; tab-width: 8 -*- */

#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "url.h"
#include "util.h"

/*
void
safe_free(void *ptr)
{
        if( ptr ) {
                debug("safe_free [%p]", ptr);
                free(ptr);
                ptr = NULL;
        }
}
*/

int
is_probably_an_url(char *string)
{
        if( strstr(string, "://") )
                return 1;

        return 0;
}

static char *
get_service_type(char *string, Url *u)
{
        /* fixme: what if the string isn't at the beginning of the 
           *string? */
        
        if( strstr(string, "http://") ) {
                string += 7;		/* skip past that part */
                u->service_type = SERVICE_HTTP;
                return string;
        }

        if( strstr(string, "ftp://") ) {
                string += 6;
                u->service_type = SERVICE_FTP;
                return string;
        }

        if( strstr(string, "gopher://") ) {
                string += 9;
                u->service_type = SERVICE_GOPHER;
                return string;
        }

        if( strncasecmp(string, "www", 3) == 0 ) {
                u->service_type = SERVICE_HTTP;
                u->full_url = strconcat("http://", u->full_url, NULL);
                return string;
        }

        if( strncasecmp(string, "ftp", 3) == 0 ) {
                u->service_type = SERVICE_FTP;
                u->full_url = strconcat("ftp://", u->full_url, NULL);
                return string;
        }

        /* default to browser-style serviceless http URL */
        u->full_url = strconcat("http://", u->full_url, NULL);
        u->service_type = SERVICE_HTTP;
        return string;
}


static char *
get_username(char *string, Url *u)
{
        int i;
        char *username;
        char *at;
        
        at = strchr(string, '@');

        if( (!at) || (at >= strchr(string, '/')) )
                return string;
        
        for( i = 0; string[i] && string[i] != ':' && string[i] != '@' &&
                     string[i] != '/'; i++);

        if( string[i] != '@' && string[i] != ':' ) {
                debug("url[i] is [%c]", string[i]);
                return string;
        }
 
        username = malloc(i);
        memcpy(username, string, i + 1);

        username[i] = '\0';

        string += i + 1;

        debug("post get_username: [%s]", string);
        u->username = username;
        return string;
}


static char *
get_password(char *string, Url *u)
{
        int i;
        char *password;
        char *at;
        
        at = strchr(string, '@');

        if( (!at) || (at >= strchr(string, '/')) )
                return string;

        /* skipping to the end of the host portion.
           this is kinda messy for the (rare) cases where someone
           has a slash and/or at in their password. It's not perfect;
           but it catches easy cases. 
           
           If you know of a better way to do this, be my guest. I do not
           feel a particular paternal instinct towards my ugly code.

           I guess that applies to this whole program.
        */

        for(i = 0 ; string[i] != '@'; i++);
        
        debug("get_password: i is %d", i);

        password = malloc(i);

        /* and finally, get the password portion */
        memcpy(password, string, i);
        password[i] = '\0';

        string += i + 1;

        u->password = password;
        
        return string;
}


static char *
get_hostname(char *url, Url *u)
{
        char *hostname;
        int i;

        /* skip to end, slash, or port colon */
        for( i = 0; url[i] && url[i] != '/' && url[i] != ':'; i++ );

        hostname = malloc(i + 1);

        memcpy(hostname, url, i);

        hostname[i] = '\0';

        /* if there's a port */
        if(url[i] == ':')
                url += i + 1;
        else
                url += i;

        u->host = hostname;
        return url;
}

static char *
get_port(char *url, Url *u)
{
        char *port_string;
        int i;

        for(i = 0; isdigit(url[i]); i++);

        if(i == 0)
                return url;


        port_string = malloc(i + 1);
        memcpy(port_string, url, i + 1);

        port_string[i] = '\0';

        url += i;
        
        u->port = atoi(port_string);

        return url;
}


static char *
get_path(char *url, Url *u)
{
        int i;
        char *path;

        debug("pre path is [%s]", url);

        /* find where the last slash is */
        for(i = strlen(url); i > 0 && url[i] != '/'; i--);

        if(url[i] != '/')
                return url;

        debug("get_path i is [%d]", i);

        path = malloc(i + 2);
        memcpy(path, url, i + 1);
        path[i] = '/';
        path[i + 1] = '\0';

        url += i + 1;
        u->path = path;

        debug("get_path path is [%s]", path);
        debug("post path is [%s]", url);
        
        return url;
}


static char *
get_file(char *string, Url *u)
{
        char *file;
        
        if( !string[0] ) 
                return NULL;

        debug("get_file: string=%s", string);

        file = malloc(strlen(string) + 1);

        memcpy(file, string, strlen(string) + 1);

        u->file = file;

        return string;
}


Url *
url_new(void)
{
	Url *new_url;

	new_url = malloc(sizeof(Url));

        new_url->full_url	= NULL;
	new_url->service_type 	= 0;
	new_url->username 	= NULL;
	new_url->password 	= NULL;
	new_url->host 		= NULL;
	new_url->port 		= 0;
	new_url->path 		= NULL;
	new_url->file		= NULL;

	return new_url;
}


void 
url_destroy(Url *u)
{
        if( !u )
                return;
        
        debug("starting url_destroy");
        debug("freeing full_url");
        safe_free(u->full_url);
        debug("freeing username");
        safe_free(u->username);
        debug("freeing password");
        safe_free(u->password);
        debug("freeing host");
        safe_free(u->host);
        debug("freeing path");
        safe_free(u->path);
        debug("freeing file");
        safe_free(u->file);
        debug("ending url_destroy");
}
        

UrlResource *
url_resource_new(void)
{
        UrlResource *new_resource;
        
        new_resource = malloc(sizeof(UrlResource));
        
        new_resource->url 		= NULL;
        new_resource->outfile		= NULL;
        new_resource->proxy		= NULL;
        new_resource->options		= 0;
        new_resource->outfile_size	= 0;
        new_resource->outfile_offset	= 0;

        return new_resource;
}


void 
url_resource_destroy(UrlResource *rsrc)
{
        if( !rsrc )
                return;

        debug("starting url_resource_destroy");

        if(rsrc->url)
                url_destroy(rsrc->url);

        debug("ending url_resource_destroy");

        safe_free(rsrc->outfile);

        free(rsrc);
}


Url *
url_init(Url *u, char *string)
{
	char *sp;	/* since we're going to walk through string,
			   use a copy instead. */

        sp = string;

        u->full_url = (char *)strdup(string);

	if( ! (sp = get_service_type(sp, u)) )
		return 0;

        debug("service type is [%d]", u->service_type);

        /* only get username/password if they are not null,
           allows us to handle redirects properly */

        if( !u->username ) 
                sp = get_username(sp, u);
        if( !u->password )
                sp = get_password(sp, u);

        sp = get_hostname(sp, u);

        if( ! (u->host && *(u->host)) )
                return NULL;

        debug("sp is [%s]", sp);

        sp = get_port(sp, u);

        debug("sp is [%s]", sp);

        sp = get_path(sp, u);
        sp = get_file(sp, u);

        debug("service=[%d] "
              "host=[%s] "
              "username=[%s] "
              "password=[%s] "
              "port=[%d] "
              "path=[%s] "
              "file=[%s] ", 
              u->service_type, u->host, u->username, u->password,
              u->port, u->path, u->file);

	return u;
}




		

